__copyright__ = """
Copyright (c) 2005, Henry Ludemann
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

 * Redistributions of source code must retain the above copyright
   notice, this list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright
   notice, this list of conditions and the following disclaimer in the
   documentation and/or other materials provided with the distribution.

 * Neither the name of the organization nor the names of its contributors
   may be used to endorse or promote products derived from this software
   without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""

import sys
import inspect
import getopt

from cmdline.Flag import Flag
from cmdline.Argument import Argument
from cmdline.CommandArgument import CommandArgument
from cmdline.MultipleLines import get_lines


class Command:
    """Class used to run a method based on command line parameters"""

    def __init__(self, method, description, post_description=""):
        """Construct a command object

        The method is the method that will be called given successful parsing,
        the description is a texture description of what the command does (used
        when printing the help), and the post_description is a description that
        follows the help (for example, ...Report bugs to blah@example.com...)
        """
        self.method         = method
        self.description    = description
        self.post_description = post_description
        self.shortflags     = ''
        self.longflags      = []
        self.flags          = []
        self.arguments      = []

        self._add_options('h', 'help', 0)
        self.flags.append(Flag('h', 'help', 'Show this help page', None))

    def add_flag(self, option, long_option, description, param_name, 
                 get_argument=None, argument_name=None):
        """Add a flag (that can have a value)

        option or long_option can be null (but not both)
        """
        has_argument = get_argument != None
        self._add_options(option, long_option, has_argument)
        self._validate_param_name(param_name)

        # Add the flag to the array 
        argument = None
        if has_argument:
            if not argument_name:
                raise ValueError, "Flag '%s' has an unnamed argument!"%\
                    (description)
            argument = Argument(argument_name, get_argument)
        flag = Flag(option, long_option, description, param_name, argument)
        self.flags.append(flag)

    def add_argument(self, name, description, param_name, get_argument=str):
        """Add an argument to be read from the command line

        Takes the name of the argument, the name of the parameter it will be
        used for, and a function to convert the argument to the correct type
        """
        self._validate_param_name(param_name)

        argument = CommandArgument(name, description, get_argument, param_name) 
        self.arguments.append(argument)

    def _add_options(self, option, long_option, hasValue):
        """Add the flags to the option lists, and ensure they are unique"""
        if option:
            if option in self.shortflags:
                raise ValueError, option + " option has allready been used!"

            shortFlag = option
            if hasValue:
                shortFlag = shortFlag + ':'
            self.shortflags = self.shortflags + shortFlag

        if long_option:
            for currentLongOption in self.longflags:
                if long_option == currentLongOption.strip('='):
                    raise ValueError, long_option + \
                        " option has allready been added!"
            longFlag = long_option
            if hasValue:
                longFlag = longFlag + "="
            self.longflags.append(longFlag)

    def _validate_param_name(self, param_name):
        "Make sure the parameter name exists"
        args, varargs, varkw, defaults = inspect.getargspec(self.method)
        if not param_name in args:
            raise ValueError, "%s isn't a valid parameter to %s! " \
                "Options are: " %(param_name, self.method.__name__) + \
                reduce(lambda a,b:a+' '+b, args)

    def _get_flag(self, flag):
        "Get a given named flag"
        for flagObj in self.flags:
            if flagObj.is_flag(flag):
                return flagObj
        raise ValueError, "Couldn't find flag %s" % (flag)

    def _get_params(self, opts, args):
        """Get a list of parameters to be used when invoking the method

        Uses the default parameter values, flag list, and arguments. Returns
        None if the parameter list couldn't be built."""
        # Create a dictionary of parameters by name to value
        values = {}
        for flag, value in opts.iteritems():
            flagObj = self._get_flag(flag)
            value = flagObj.get_value(value)
            if value == None:
                print "Failed to get value for flag", str(flagObj)
                return None
            values[flagObj.get_param_name()] = value

        # Add all arguments to the dictionary
        if len(args) > len(self.arguments):
            print "Too many arguments! Expected %i got %i" % \
                (len(self.arguments), len(args))
            return None
        elif len(args) < len(self.arguments):
            print "Not enough arguments! Missing",
            for argumentIndex in range(len(args), len(self.arguments)):
                print " ", self.arguments[argumentIndex].get_name(),
            print
            return None

        for arg_index in range(len(args)):
            argument = self.arguments[arg_index]
            value = argument.get_value(args[arg_index])
            if value == None:
                print "Failed to get value for argument", argument.get_name()
                return None
            values[argument.get_param_name()] = value

        # Create a list of method parameter values, using the named value
        # if it exists, otherwise using the default parameter
        method_args, varargs, varkw, defaults = inspect.getargspec(self.method)
        if defaults == None:
            # When there are no default parameters, we want to handle it
            # correctly
            defaults = []

        parameters = []
        for arg_index in range(len(method_args)): 
            arg = method_args[arg_index]
            if arg_index == 0 and inspect.ismethod(self.method):
                # The first value is the instance of the object, and as such
                # we don't need to override it (call the method object fills
                # in this value)
                pass
            elif values.has_key(arg):
                parameters.append(values[arg])
            else:
                defaultIndex = arg_index - (len(method_args) - len(defaults))
                if defaultIndex < 0:
                    raise ValueError, "No value for parameter %s when " \
                        "invoking %s"% \
                        (method_args[arg_index], self.method.__name__)
                parameters.append(defaults[defaultIndex])
        return parameters

    def _display_usage(self, command):
        """Print the usage of the command"""
        output = "Usage: %s [OPTIONS]"%(command)
        appendArgument = lambda a,b:"%s %s"%(a, b.get_name())
        output = reduce(appendArgument, self.arguments, output)
        print output
        print get_lines( self.description),
        print
        if len(self.arguments) > 0:
            print "Arguments"
            for argument in self.arguments:
                arg = "  %s "%(argument.get_name())
                print arg + get_lines(argument.description, offset = 24, 
                                      first_offset = len(arg)),
            print
        print "Mandatory arguments to long flags are mandatory for " \
              "short options too"
        # Sort the flags alphabetically, then print them out
        sort_flags = lambda a,b: a.get_flag_name() < b.get_flag_name()
        flags = self.flags[:]
        flags.sort(sort_flags)
        for flag in flags:
             flag_str = "  %s"%(str(flag))
             print flag_str + get_lines(flag.description, offset=24, 
                                        first_offset = len(flag_str)) ,

        if len(self.post_description) > 0:
            print
            print self.post_description

    def invoke(self, command=sys.argv[0], arguments=sys.argv[1:]): 
        """Run the command with the given arguments.

        If an error with the parsing occurs, it prints an error before
        returning with a value of 1. If the target function is called (if
        the parameters parsed correctly) that return value will be returned.
        The command variable is used to print a useful 'usage' line.
        """
        try:
            opts, args = getopt.getopt(arguments, self.shortflags, 
                                       self.longflags)
        except getopt.GetoptError, errorInfo:
            # print help information and exit:
            print "Error running", command, 
            print reduce(lambda a,b:a + ' ' + b, arguments)
            print errorInfo
            print "Run", command, "--help for more information"
            return 1

        # Change the options to a dictionary, as this is more useful. Note
        # that this inhibits the reuse of flags.
        opts = dict(opts)

        # Do a special check for the internal help flags
        if opts.has_key('-h') or opts.has_key('--help'):
            if len(opts) > 1 or len(args) > 0:
                print "Can only use -h or --help flags by themselves!"
                return 1
            self._display_usage(command)
            return 0

        # Get the parameters from the command line arguments
        params = self._get_params(opts, args)
        if params == None: 
            print "Run", command, "--help for more information"
            return 1

        result = self.method(*params)
        return result
